@@ -12,8 +12,25 @@
 
 #include "soup-headers.h"
 #include "soup-misc.h"
+#include "soup-uri.h"
 
-static gboolean
+/**
+ * soup_headers_parse:
+ * @str: the header string (including the Request-Line or Status-Line,
+ * and the trailing blank line)
+ * @len: length of @str up to (but not including) the terminating blank line.
+ * @dest: #SoupMessageHeaders to store the header values in
+ *
+ * Parses the headers of an HTTP request or response in @str and
+ * stores the results in @dest. Beware that @dest may be modified even
+ * on failure.
+ *
+ * This is a low-level method; normally you would use
+ * soup_headers_parse_request() or soup_headers_parse_response().
+ *
+ * Return value: success or failure
+ **/
+gboolean
 soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
 {
 	const char *headers_start;
@@ -367,23 +384,23 @@ unskip_lws (const char *s, const char *start)
 }
 
 static const char *
-skip_commas (const char *s)
+skip_delims (const char *s, char delim)
 {
-	/* The grammar allows for multiple commas */
-	while (g_ascii_isspace (*s) || *s == ',')
+	/* The grammar allows for multiple delimiters */
+	while (g_ascii_isspace (*s) || *s == delim)
 		s++;
 	return s;
 }
 
 static const char *
-skip_item (const char *s)
+skip_item (const char *s, char delim)
 {
 	gboolean quoted = FALSE;
 	const char *start = s;
 
 	/* A list item ends at the last non-whitespace character
-	 * before a comma which is not inside a quoted-string. Or at
-	 * the end of the string.
+	 * before a delimiter which is not inside a quoted-string. Or
+	 * at the end of the string.
 	 */
 
 	while (*s) {
@@ -393,7 +410,7 @@ skip_item (const char *s)
 			if (*s == '\\' && *(s + 1))
 				s++;
 		} else {
-			if (*s == ',')
+			if (*s == delim)
 				break;
 		}
 		s++;
@@ -402,6 +419,22 @@ skip_item (const char *s)
 	return unskip_lws (s, start);
 }
 
+static GSList *
+parse_list (const char *header, char delim)
+{
+	GSList *list = NULL;
+	const char *end;
+
+	header = skip_delims (header, delim);
+	while (*header) {
+		end = skip_item (header, delim);
+		list = g_slist_prepend (list, g_strndup (header, end - header));
+		header = skip_delims (end, delim);
+	}
+
+	return g_slist_reverse (list);
+}
+
 /**
  * soup_header_parse_list:
  * @header: a header value
@@ -415,17 +448,7 @@ skip_item (const char *s)
 GSList *
 soup_header_parse_list (const char *header)
 {
-	GSList *list = NULL;
-	const char *end;
-
-	header = skip_commas (header);
-	while (*header) {
-		end = skip_item (header);
-		list = g_slist_prepend (list, g_strndup (header, end - header));
-		header = skip_commas (end);
-	}
-
-	return g_slist_reverse (list);
+	return parse_list (header, ',');
 }
 
 typedef struct {
@@ -569,13 +592,13 @@ soup_header_contains (const char *header, const char *token)
 	const char *end;
 	guint len = strlen (token);
 
-	header = skip_commas (header);
+	header = skip_delims (header, ',');
 	while (*header) {
-		end = skip_item (header);
+		end = skip_item (header, ',');
 		if (end - header == len &&
 		    !g_ascii_strncasecmp (header, token, len))
 			return TRUE;
-		header = skip_commas (end);
+		header = skip_delims (end, ',');
 	}
 
 	return FALSE;
@@ -596,26 +619,37 @@ decode_quoted_string (char *quoted_string)
 	*dst = '\0';
 }
 
-/**
- * soup_header_parse_param_list:
- * @header: a header value
- *
- * Parses a header which is a list of something like
- *   token [ "=" ( token | quoted-string ) ]
- *
- * Tokens that don't have an associated value will still be added to
- * the resulting hash table, but with a %NULL value.
- * 
- * Return value: a #GHashTable of list elements.
- **/
-GHashTable *
-soup_header_parse_param_list (const char *header)
+static gboolean
+decode_rfc2231 (char *encoded_string)
+{
+	char *q, *decoded;
+
+	q = strchr (encoded_string, '\'');
+	if (!q)
+		return FALSE;
+	if (g_ascii_strncasecmp (encoded_string, "UTF-8",
+				 q - encoded_string) != 0)
+		return FALSE;
+
+	q = strchr (q + 1, '\'');
+	if (!q)
+		return FALSE;
+
+	decoded = soup_uri_decode (q + 1);
+	/* strlen(decoded) <= strlen(q + 1) < strlen(encoded_string) */
+	strcpy (encoded_string, decoded);
+	g_free (decoded);
+	return TRUE;
+}
+
+static GHashTable *
+parse_param_list (const char *header, char delim)
 {
 	GHashTable *params;
 	GSList *list, *iter;
 	char *item, *eq, *name_end, *value;
 
-	list = soup_header_parse_list (header);
+	list = parse_list (header, delim);
 	if (!list)
 		return NULL;
 
@@ -638,7 +672,14 @@ soup_header_parse_param_list (const char *header)
 			*name_end = '\0';
 
 			value = (char *)skip_lws (eq + 1);
-			if (*value == '"')
+
+			if (name_end[-1] == '*' && name_end > item + 1) {
+				name_end[-1] = '\0';
+				if (!decode_rfc2231 (value)) {
+					g_free (item);
+					continue;
+				}
+			} else if (*value == '"')
 				decode_quoted_string (value);
 		} else
 			value = NULL;
@@ -650,9 +691,56 @@ soup_header_parse_param_list (const char *header)
 	return params;
 }
 
+/**
+ * soup_header_parse_param_list:
+ * @header: a header value
+ *
+ * Parses a header which is a comma-delimited list of something like:
+ * <literal>token [ "=" ( token | quoted-string ) ]</literal>.
+ *
+ * Tokens that don't have an associated value will still be added to
+ * the resulting hash table, but with a %NULL value.
+ * 
+ * This also handles RFC2231 encoding (which in HTTP is mostly used
+ * for giving UTF8-encoded filenames in the Content-Disposition
+ * header).
+ *
+ * Return value: a #GHashTable of list elements, which can be freed
+ * with soup_header_free_param_list().
+ **/
+GHashTable *
+soup_header_parse_param_list (const char *header)
+{
+	return parse_param_list (header, ',');
+}
+
+/**
+ * soup_header_parse_semi_param_list:
+ * @header: a header value
+ *
+ * Parses a header which is a semicolon-delimited list of something
+ * like: <literal>token [ "=" ( token | quoted-string ) ]</literal>.
+ *
+ * Tokens that don't have an associated value will still be added to
+ * the resulting hash table, but with a %NULL value.
+ * 
+ * This also handles RFC2231 encoding (which in HTTP is mostly used
+ * for giving UTF8-encoded filenames in the Content-Disposition
+ * header).
+ *
+ * Return value: a #GHashTable of list elements, which can be freed
+ * with soup_header_free_param_list().
+ **/
+GHashTable *
+soup_header_parse_semi_param_list (const char *header)
+{
+	return parse_param_list (header, ';');
+}
+
 /**
  * soup_header_free_param_list:
  * @param_list: a #GHashTable returned from soup_header_parse_param_list()
+ * or soup_header_parse_semi_param_list()
  *
  * Frees @param_list.
  **/
@@ -661,3 +749,67 @@ soup_header_free_param_list (GHashTable *param_list)
 {
 	g_hash_table_destroy (param_list);
 }
+
+static void
+append_param_rfc2231 (GString *string, const char *value)
+{
+	char *encoded;
+
+	g_string_append (string, "*=UTF-8''");
+	encoded = soup_uri_encode (value, " *'%()<>@,;:\\\"/[]?=");
+	g_string_append (string, encoded);
+	g_free (encoded);
+}
+
+static void
+append_param_quoted (GString *string, const char *value)
+{
+	int len;
+
+	g_string_append (string, "=\"");
+	while (*value) {
+		while (*value == '\\' || *value == '"') {
+			g_string_append_c (string, '\\');
+			g_string_append_c (string, *value++);
+		}
+		len = strcspn (value, "\\\"");
+		g_string_append_len (string, value, len);
+		value += len;
+	}
+	g_string_append_c (string, '"');
+}
+
+/**
+ * soup_header_g_string_append_param:
+ * @string: a #GString being used to construct an HTTP header value
+ * @name: a parameter name
+ * @value: a parameter value
+ *
+ * Appends something like <literal>@name="@value"</literal> to @string,
+ * taking care to appropriately escape any quotes or backslashes in @value.
+ *
+ * Alternatively, if @value is a non-ASCII UTF-8 string, it will be
+ * appended using RFC2231 syntax. Although in theory this is supposed
+ * to work anywhere in HTTP that uses this style of parameter, in
+ * reality, it can only be used portably with the Content-Disposition
+ * "filename" parameter.
+ **/
+void
+soup_header_g_string_append_param (GString *string, const char *name,
+				   const char *value)
+{
+	const char *v;
+
+	g_string_append (string, name);
+
+	for (v = value; *v; v++) {
+		if (*v & 0x80) {
+			if (g_utf8_validate (value, -1, NULL)) {
+				append_param_rfc2231 (string, value);
+				return;
+			} else
+				break;
+		}
+	}
+	append_param_quoted (string, value);
+}
