@@ -9,6 +9,7 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -28,6 +29,10 @@
  * SECTION:soup-socket
  * @short_description: A network socket
  *
+ * #SoupSocket is libsoup's TCP socket type. While it is primarily
+ * intended for internal use, #SoupSocket<!-- -->s are exposed in the
+ * API in various places, and some of their methods (eg,
+ * soup_socket_get_remote_address()) may be useful to applications.
  **/
 
 G_DEFINE_TYPE (SoupSocket, soup_socket, G_TYPE_OBJECT)
@@ -158,6 +163,10 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (socket_class);
 
+#ifdef SIGPIPE
+	signal (SIGPIPE, SIG_IGN);
+#endif
+
 	g_type_class_add_private (socket_class, sizeof (SoupSocketPrivate));
 
 	/* virtual method override */
@@ -324,8 +333,8 @@ static void
 set_fdflags (SoupSocketPrivate *priv)
 {
 	int opt;
-	struct timeval timeout;
 #ifndef G_OS_WIN32
+	struct timeval timeout;
 	int flags;
 #endif
 
@@ -348,6 +357,7 @@ set_fdflags (SoupSocketPrivate *priv)
 	setsockopt (priv->sockfd, SOL_SOCKET,
 		    SO_REUSEADDR, (void *) &opt, sizeof (opt));
 
+#ifndef G_OS_WIN32
 	timeout.tv_sec = priv->timeout;
 	timeout.tv_usec = 0;
 	setsockopt (priv->sockfd, SOL_SOCKET,
@@ -357,6 +367,18 @@ set_fdflags (SoupSocketPrivate *priv)
 	timeout.tv_usec = 0;
 	setsockopt (priv->sockfd, SOL_SOCKET,
 		    SO_SNDTIMEO, (void *) &timeout, sizeof (timeout));
+#else
+	if (priv->timeout < G_MAXINT / 1000)
+		opt = priv->timeout * 1000;
+	else
+		opt = 0;
+
+	setsockopt (priv->sockfd, SOL_SOCKET,
+		    SO_RCVTIMEO, (void *) &opt, sizeof (opt));
+	
+	setsockopt (priv->sockfd, SOL_SOCKET,
+		    SO_SNDTIMEO, (void *) &opt, sizeof (opt));
+#endif
 
 #ifndef G_OS_WIN32
 	priv->iochannel =
@@ -399,6 +421,7 @@ set_property (GObject *object, guint prop_id,
 		priv->timeout = g_value_get_uint (value);
 		break;
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -432,6 +455,7 @@ get_property (GObject *object, guint prop_id,
 		g_value_set_uint (value, priv->timeout);
 		break;
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -538,8 +562,8 @@ async_cancel (GCancellable *cancellable, gpointer user_data)
 	if (priv->watch_src)
 		g_source_destroy (priv->watch_src);
 	disconnect_internal (priv);
-	priv->watch_src = soup_add_idle (priv->async_context,
-					 idle_connect_result, sacd);
+	priv->watch_src = soup_add_completion (priv->async_context,
+					       idle_connect_result, sacd);
 }
 
 static guint
@@ -637,8 +661,8 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 						  sacd);
 		}
 	} else {
-		priv->watch_src = soup_add_idle (priv->async_context,
-						 idle_connect_result, sacd);
+		priv->watch_src = soup_add_completion (priv->async_context,
+						       idle_connect_result, sacd);
 	}
 }
 
@@ -846,7 +870,7 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 
 	real_chan = priv->iochannel;
 	ssl_chan = soup_ssl_wrap_iochannel (
-		real_chan, priv->is_server ?
+		real_chan, priv->non_blocking, priv->is_server ?
 		SOUP_SSL_TYPE_SERVER : SOUP_SSL_TYPE_CLIENT,
 		ssl_host, priv->ssl_creds);
 
@@ -859,6 +883,16 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 	return TRUE;
 }
 	
+/**
+ * soup_socket_is_ssl:
+ * @sock: a #SoupSocket
+ *
+ * Tests if @sock is set up to do SSL. Note that this simply means
+ * that the %SOUP_SOCKET_SSL_CREDENTIALS property has been set; it
+ * does not mean that soup_socket_start_ssl() has been called.
+ *
+ * Return value: %TRUE if @sock has SSL credentials set
+ **/
 gboolean
 soup_socket_is_ssl (SoupSocket *sock)
 {
@@ -1026,7 +1060,9 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 	GIOCondition cond = G_IO_IN;
 	GError *my_err = NULL;
 
-	if (!priv->iochannel) 
+	*nread = 0;
+
+	if (!priv->iochannel)
 		return SOUP_SOCKET_EOF;
 
 	status = g_io_channel_read_chars (priv->iochannel,
@@ -1137,6 +1173,8 @@ soup_socket_read (SoupSocket *sock, gpointer buffer, gsize len,
 	SoupSocketIOStatus status;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_SOCKET_ERROR);
+	g_return_val_if_fail (nread != NULL, SOUP_SOCKET_ERROR);
+
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	g_mutex_lock (priv->iolock);
@@ -1182,9 +1220,11 @@ soup_socket_read_until (SoupSocket *sock, gpointer buffer, gsize len,
 	guint8 *p, *end;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_SOCKET_ERROR);
-	priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	g_return_val_if_fail (nread != NULL, SOUP_SOCKET_ERROR);
 	g_return_val_if_fail (len >= boundary_len, SOUP_SOCKET_ERROR);
 
+	priv = SOUP_SOCKET_GET_PRIVATE (sock);
+
 	g_mutex_lock (priv->iolock);
 
 	*got_boundary = FALSE;
@@ -1255,7 +1295,7 @@ socket_write_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
  *
  * Attempts to write @len bytes from @buffer to @sock. If some data is
  * successfully written, the resturn status will be
- * %SOUP_SOCKET_SUCCESS, and *@nwrote will contain the number of bytes
+ * %SOUP_SOCKET_OK, and *@nwrote will contain the number of bytes
  * actually written.
  *
  * If @sock is non-blocking, and no data could be written right away,
@@ -1275,13 +1315,12 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 {
 	SoupSocketPrivate *priv;
 	GIOStatus status;
-#ifdef SIGPIPE
-	gpointer pipe_handler;
-#endif
 	GIOCondition cond = G_IO_OUT;
 	GError *my_err = NULL;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_SOCKET_ERROR);
+	g_return_val_if_fail (nwrote != NULL, SOUP_SOCKET_ERROR);
+
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	g_mutex_lock (priv->iolock);
@@ -1295,14 +1334,8 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 		return SOUP_SOCKET_WOULD_BLOCK;
 	}
 
-#ifdef SIGPIPE
-	pipe_handler = signal (SIGPIPE, SIG_IGN);
-#endif
 	status = g_io_channel_write_chars (priv->iochannel,
 					   buffer, len, nwrote, &my_err);
-#ifdef SIGPIPE
-	signal (SIGPIPE, pipe_handler);
-#endif
 	if (my_err) {
 		if (my_err->domain == SOUP_SSL_ERROR &&
 		    my_err->code == SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ)
