@@ -8,23 +8,7 @@
 #include <unistd.h>
 
 #include "libsoup/soup.h"
-#include "apache-wrapper.h"
-
-int errors = 0;
-gboolean debug = FALSE;
-
-static void
-dprintf (const char *format, ...)
-{
-	va_list args;
-
-	if (!debug)
-		return;
-
-	va_start (args, format);
-	vprintf (format, args);
-	va_end (args);
-}
+#include "test-utils.h"
 
 typedef struct {
 	const char *explanation;
@@ -32,14 +16,14 @@ typedef struct {
 	const guint final_status;
 } SoupProxyTest;
 
-SoupProxyTest tests[] = {
+static SoupProxyTest tests[] = {
 	{ "GET -> 200", "", SOUP_STATUS_OK },
 	{ "GET -> 404", "/not-found", SOUP_STATUS_NOT_FOUND },
 	{ "GET -> 401 -> 200", "/Basic/realm1/", SOUP_STATUS_OK },
 	{ "GET -> 401 -> 401", "/Basic/realm2/", SOUP_STATUS_UNAUTHORIZED },
-	{ "GET -> 403", "http://no-proxy.example.com/", SOUP_STATUS_FORBIDDEN },
+	{ "GET -> 403", "http://www.example.com/", SOUP_STATUS_FORBIDDEN },
 };
-int ntests = sizeof (tests) / sizeof (tests[0]);
+static int ntests = sizeof (tests) / sizeof (tests[0]);
 
 #define HTTP_SERVER    "http://127.0.0.1:47524"
 #define HTTPS_SERVER   "https://127.0.0.1:47525"
@@ -62,21 +46,20 @@ static const char *proxy_names[] = {
 
 static void
 authenticate (SoupSession *session, SoupMessage *msg,
-	      const char *auth_type, const char *auth_realm,
-	      char **username, char **password, gpointer data)
+	      SoupAuth *auth, gboolean retrying, gpointer data)
 {
-	*username = g_strdup ("user1");
-	*password = g_strdup ("realm1");
+	if (!retrying)
+		soup_auth_authenticate (auth, "user1", "realm1");
 }
 
 static void
 test_url (const char *url, int proxy, guint expected, gboolean sync)
 {
 	SoupSession *session;
-	SoupUri *proxy_uri;
+	SoupURI *proxy_uri;
 	SoupMessage *msg;
 
-	dprintf ("  GET %s via %s\n", url, proxy_names[proxy]);
+	debug_printf (1, "  GET %s via %s\n", url, proxy_names[proxy]);
 	if (proxy == UNAUTH_PROXY && expected != SOUP_STATUS_FORBIDDEN)
 		expected = SOUP_STATUS_PROXY_UNAUTHORIZED;
 
@@ -99,9 +82,9 @@ test_url (const char *url, int proxy, guint expected, gboolean sync)
 
 	soup_session_send_message (session, msg);
 
-	dprintf ("  %d %s\n", msg->status_code, msg->reason_phrase);
+	debug_printf (1, "  %d %s\n", msg->status_code, msg->reason_phrase);
 	if (msg->status_code != expected) {
-		dprintf ("  EXPECTED %d!\n", expected);
+		debug_printf (1, "  EXPECTED %d!\n", expected);
 		errors++;
 	}
 
@@ -115,8 +98,8 @@ run_test (int i, gboolean sync)
 {
 	char *http_url, *https_url;
 
-	dprintf ("Test %d: %s (%s)\n", i + 1, tests[i].explanation,
-		 sync ? "sync" : "async");
+	debug_printf (1, "Test %d: %s (%s)\n", i + 1, tests[i].explanation,
+		      sync ? "sync" : "async");
 
 	if (!strncmp (tests[i].url, "http", 4)) {
 		http_url = g_strdup (tests[i].url);
@@ -141,45 +124,22 @@ run_test (int i, gboolean sync)
 	g_free (http_url);
 	g_free (https_url);
 
-	dprintf ("\n");
+	debug_printf (1, "\n");
 }
 
 int
 main (int argc, char **argv)
 {
-	int i, opt;
+	int i;
 
-	g_type_init ();
-	g_thread_init (NULL);
-
-	while ((opt = getopt (argc, argv, "d")) != -1) {
-		switch (opt) {
-		case 'd':
-			debug = TRUE;
-			break;
-		default:
-			fprintf (stderr, "Usage: %s [-d]\n", argv[0]);
-			return 1;
-		}
-	}
-
-	if (!apache_init ()) {
-		fprintf (stderr, "Could not start apache\n");
-		return 1;
-	}
+	test_init (argc, argv, NULL);
+	apache_init ();
 
 	for (i = 0; i < ntests; i++) {
 		run_test (i, FALSE);
 		run_test (i, TRUE);
 	}
 
-	apache_cleanup ();
-
-	dprintf ("\n");
-	if (errors) {
-		printf ("proxy-test: %d error(s). Run with '-d' for details\n",
-			errors);
-	} else
-		printf ("proxy-test: OK\n");
-	return errors;
+	test_cleanup ();
+	return errors != 0;
 }
